import click
from .releaser import ResinReleaser


# get releaser object from the context
pass_releaser = click.make_pass_decorator(ResinReleaser)


@click.group()
@click.option('--token', required=True, envvar='RESIN_TOKEN',
              metavar='TOKEN', help='Resin.io auth token')
@click.option('--app', required=True, envvar='RESIN_APP',
              metavar='APP_ID', help='Resin App name')
@click.pass_context
def cli(ctx, app, token):
    """You can set app and token as environment variables,
    using RESIN_APP and RESIN_TOKEN
    """
    ctx.obj = ResinReleaser(token, app)


@cli.command()
@pass_releaser
def info(releaser):
    """Information of the application"""

    info = releaser.get_info()
    rolling = info['should_track_latest_release'] and 'Yes' or 'No'
    info_list = [
        f"App Name: {info['app_name']}",
        f"Device Type: {info['device_type']}",
        f"In Commit: {info['commit']}",
        f"Rolling enabled: {rolling}"]
    click.echo('\n'.join(info_list))


@cli.command()
@pass_releaser
def disable_rolling(releaser):
    """Disables rolling releases in the application"""
    releaser.disable_rolling()
    click.echo("Disabled rolling")


@cli.command()
@pass_releaser
def enable_rolling(releaser):
    """Enables rolling releases in the application"""
    releaser.enable_rolling()
    releases = list(releaser.get_releases().values())
    if releases:
        commit = releases[0]['commit']
        releaser.set_app_to_release(commit)
    click.echo("Enabled rolling")


@cli.command()
@pass_releaser
def show_devices_status(releaser):
    """Show the status of the devices in the applications"""

    devices = releaser.get_devices_by_status()

    canaries = ', '.join(
        [c['uuid'][:6] for c in devices['canaries'].values()])
    old_canaries = ', '.join(
        [c['uuid'][:6] for c in devices['old_canaries'].values()])

    rest = list(devices['rest'].values())
    rest_len = len(rest)
    rest_info = ', '.join([c['uuid'][:6] for c in rest[:10]])
    if rest_len > 10:
        rest_info += f'... and {rest_len-10} more'

    click.echo(f'Canaries: {canaries}')
    click.echo(f'Old Canaries: {old_canaries}')
    click.echo(f'Rest of the Devices: {rest_info}')


@cli.command()
@click.argument('release_commit')
@click.argument('canary_commit')
@pass_releaser
@click.pass_context
def release(ctx, releaser, release_commit, canary_commit):
    """Sets release and canary commits"""
    if not releaser.is_valid_commit(release_commit):
        click.echo(f'Invalid release commit: {release_commit}')
        exit(2)
    if not releaser.is_valid_commit(canary_commit):
        click.echo('Invalid canary commit: {canary_commit}')
        exit(2)

    ctx.invoke(info)
    click.echo('Devices:')
    ctx.invoke(show_devices_status)
    click.echo()

    confirm_text = 'Are you sure you want to set '\
        'release/canary to: "%s" "%s"?' % (
            release_commit, canary_commit)
    if not click.confirm(confirm_text):
        click.echo('Cancelled!')
        exit(1)
    releaser.set_release(release_commit, canary_commit)


@cli.command()
@click.option('--count', default=10, help='How many')
@pass_releaser
def releases(releaser, count):
    """Show successful releases of the application"""
    releases = list(releaser.get_releases().values())
    click.echo(f'Latest {count} releases:')
    for release in releases[:count]:
        click.echo(f'{release["end_timestamp"]} {release["commit"]}')


if __name__ == '__main__':
    cli()



from functools import lru_cache
from resin import Resin


class ResinReleaser:
    def __init__(self, token, app_id):
        self.resin = Resin()
        self.resin.auth.login_with_token(token)

        self.models = self.resin.models

        self.app_id = app_id

    def get_info(self):
        return self.models.application.get_by_id(self.app_id)

    def get_canaries(self):
        tags = self.models.tag.device.get_all_by_application(self.app_id)
        canaries = [
            tag['device']['__id'] for tag in tags
            if tag['tag_key'] == 'CANARY']
        return canaries

    @lru_cache()
    def get_releases(self):
        releases = self.models.release.get_all_by_application(self.app_id)
        return {
            release['commit']: release
            for release in releases
            if release['status'] == 'success'}

    def is_valid_commit(self, commit):
        return commit in self.get_releases()

    def disable_rolling(self):
        self.models.application.disable_rolling_updates(self.app_id)

    def enable_rolling(self):
        self.models.application.enable_rolling_updates(self.app_id)

    def set_app_to_release(self, release):
        self.models.application.set_to_release(self.app_id, release)

    def set_device_to_release(self, device, release):
        uuid = device['uuid']
        self.models.device.set_to_release(uuid, release)

    def get_all_devices(self):
        devices = self.models.device.get_all_by_application_id(self.app_id)
        return {d['id']: d for d in devices}

    @lru_cache()
    def get_devices_by_status(self):
        """Group devices by status: canary, old_canary, rest
        """
        all_devices = self.get_all_devices()
        canaries = {c: all_devices[c] for c in self.get_canaries()}

        def not_canary(device):
            return device['id'] not in canaries and \
                    device['should_be_running__release']

        old_canaries = {device['id']: device for device in all_devices.values()
                        if not_canary(device)}

        rest = {
          device['id']: device for device in all_devices.values()
          if device['id'] not in canaries and device['id'] not in old_canaries
        }

        return {
            'canaries': canaries,
            'old_canaries': old_canaries,
            'rest': rest,
        }

    def set_release(self, release_hash, canary_hash=None):
        devices = self.get_devices_by_status()

        canaries = devices['canaries']
        old_canaries = devices['old_canaries']

        # Disable rolling releases
        print('Disabling rolling releases on the application')
        self.disable_rolling()

        if old_canaries:
            print('Reseting all canaries')
            # Reset old canaries to app release
            for old_canary in old_canaries.values():
                print(old_canary['device_name'])
                self.set_device_to_release(old_canary, None)

        if canaries:
            print('Setting canaries')
            # Set canaries to current canary release
            for canary in canaries.values():
                print(canary['device_name'])
                self.set_device_to_release(canary, canary_hash)

        # We do this here to trigger update in all devices
        print('Setting up current release to: %s' % release_hash)
        self.set_app_to_release(release_hash)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['resin_release_tool']

package_data = \
{'': ['*']}

install_requires = \
['click>=6.7,<7.0', 'resin-sdk>=5.0,<6.0']

entry_points = \
{'console_scripts': ['resin-release-tool = resin_release_tool.cli:cli']}

setup_kwargs = {
    'name': 'resin-release-tool',
    'version': '0.1.0',
    'description': 'Release tool to have canary in resin.io',
    'long_description': '# resin-release-tool\nThis tool is to set release canary in resin.io\n\n## Installation\n```\npip install resin-release-tool\n```\n\n## Build / Run locally\nYou need poetry to build the project https://poetry.eustace.io/\n```\npoetry install\npoetry build\npoetry run resin-release-tool\netc..\n```\n\n\n## Usage\n```\nUsage: resin-release-tool [OPTIONS] COMMAND [ARGS]...\n\n  You can set app and token as an environment variable, using RESIN_APP and\n  RESIN_TOKEN\n\nOptions:\n  --token TOKEN  Resin.io auth token  [required]\n  --app APP_ID   Resin App name  [required]\n  --help         Show this message and exit.\n\nCommands:\n  disable_rolling      Disables rolling releases in the application\n  enable_rolling       Enables rolling releases in the application\n  info                 Information of the application\n  release              Sets release and canary commits\n  releases             Show success releases of the application\n  show_devices_status  Enables rolling releases in the application\n```\n',
    'author': 'roger',
    'author_email': 'roger.duran@mobilityhouse.com',
    'url': 'https://github.com/mobilityhouse/resin-release-tool',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
